@@ -12,7 +12,7 @@ from Tkinter import *
 from command_seq_reader import last_assignment_or_evaluatable
 import pyparsing
 import time, threading, functools, string, optparse, sys
-__version__ = '0.1.1'
+__version__ = '0.1.2'
 
 optparser = optparse.OptionParser()
 optparser.add_option("-n", "--num-targets", help="Number of target panes to display",
@@ -22,10 +22,25 @@ optparser.add_option("-l", "--lines", help="Height of grammar pane (in lines)",
 optparser.add_option("-w", "--width", help="Width of pane (in characters)",
                      dest="grammar_width", default=40, action="store", type="int")
 
+def _eq_monkeypatch(self, other):
+    if isinstance(other, pyparsing.ParserElement):
+        return self.__dict__ == other.__dict__
+    elif isinstance(other, basestring):
+        try:
+            (self + StringEnd()).parseString(_ustr(other))
+            return True
+        except ParseBaseException:
+            return False
+    else:
+        return super(ParserElement,self)==other
+
+pyparsing.ParserElement.__eq__ = _eq_monkeypatch
+
 class Application(Frame):
     grammar_height=40
     recalc_lag = 3.0
-    def __init__(self, master=None, grammar_width=40, grammar_height=40, num_targets=3):
+    def __init__(self, master=None, grammar_width=40, grammar_height=40, num_targets=3,
+                 grammar = None):
         Frame.__init__(self, master)
         self.grid()
         self.recalc_timer = None
@@ -33,6 +48,9 @@ class Application(Frame):
         self.grammar_width = grammar_width
         self.num_targets = num_targets
         self.createWidgets()
+        if grammar:
+            self.grammar_text.insert(END, grammar)
+        
     def restart_timer(self):
         if self.recalc_timer:
             self.recalc_timer.cancel()
@@ -84,7 +102,11 @@ class Application(Frame):
                 for i in range(self.num_targets):
                     self.apply_grammar(i)
             except Exception, e:
-                self.set_all_results(str(e))
+                if hasattr(e, 'text'):
+                    errtxt = '%s\n\n%s' % (str(e), e.text)
+                else:
+                    errtxt = str(e)
+                self.set_all_results(errtxt)
         else:
             self.set_all_results('')
         self.restart_timer()
@@ -144,9 +166,14 @@ class Application(Frame):
           
 def main():
     (opts, args) = optparser.parse_args()
-    app = Application(**opts.__dict__)
+    if args:
+        grammarfile = open(args[0])
+        grammar = grammarfile.read()
+    else:
+        grammar = ''
+    app = Application(grammar=grammar, **opts.__dict__)
     app.master.title("PyParsing helper") 
     app.mainloop()
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
